'''
By Michael Trikosko
www.MichaelTrikosko.com
version 1.1
- Removed reload(script) line from the hotkey to allow it to work for maya 2022+
version 1.0
- Initial release
'''

import maya.cmds as mc
import maya.mel as mel

class tweenClass():

    def refreshSelection(self):

        selection = mc.ls(sl=True, type='transform')

        self.currentTime = mc.currentTime(q=True)

        self.dictionary = {}
        if selection:
            attributes = mc.channelBox("mainChannelBox", q=True, sma=True)
            if attributes:
                curves = []
                for attr in attributes:
                    for node in selection:
                        fullnode = '{}.{}'.format(node, attr)
                        if not mc.objExists(fullnode):
                            continue
                        tmp = mc.keyframe(fullnode, q=True, name=True)
                        if not tmp:
                            continue
                        curves += tmp
            else:
                curves = mc.keyframe(selection, q=True, name=True)
            if curves:
                for curve in curves:
                    prvKey = mc.findKeyframe(curve, which="previous")
                    nxtKey = mc.findKeyframe(curve, which="next")
                    prvValue = mc.keyframe(curve, time=(prvKey,), q=True, valueChange=True)[0]
                    nxtValue = mc.keyframe(curve, time=(nxtKey,), q=True, valueChange=True)[0]

                    self.dictionary[curve] = (prvValue, nxtValue)

    def settings(self, barLength, barOffset):
        self.barLength = barLength
        self.barOffset = barOffset

    def tween(self, percentage, scrub):

        message = self.textDraw(percentage)
        mc.headsUpMessage(message, verticalOffset=self.barOffset)
        if scrub == True:
            for curve in self.dictionary.keys():
                prvValue = self.dictionary[curve][0]
                nxtValue = self.dictionary[curve][1]

                if prvValue == None or nxtValue == None:
                    continue

                difference = (nxtValue - prvValue) / 2
                midpoint = (nxtValue + prvValue) / 2
                newValue = midpoint + difference * percentage / 100

                mc.setKeyframe(curve, value=newValue)

    def textDraw(self, percentage):
        percentage = percentage / 100
        if percentage == 0:
            percentString = '___'
        else:
            percentString = str(int(abs(percentage * 100)))
        while len(percentString) < 3:
            percentString = '_' + percentString

        barLengthPercent = self.barLength * percentage

        nextBar = ''
        for i in range(0, self.barLength):
            if i > barLengthPercent:
                nextBar += '-'
            else:
                nextBar += ']'

        prevBar = ''
        for i in range(0, self.barLength):
            if i > barLengthPercent * -1:
                prevBar += '-'
            else:
                prevBar += '['

        barTickSize = 100 / self.barLength
        overshootTickAmmount = int(((abs(percentage) * 100) - 100) / barTickSize)

        if percentage > 1:
            for i in range(0, overshootTickAmmount):
                nextBar += '|'
                prevBar += '-'
        if percentage < -1:
            for i in range(0, overshootTickAmmount):
                nextBar += '-'
                prevBar += '|'

        prevBar = prevBar[::-1]

        return '{}  {}  {}'.format(prevBar, percentString, nextBar)


class dragTween():

    def __init__(self):
        self.nestedClass = tweenClass()
        self.previousDistance = 0
        self.subtract = 0
        self.add = 0
        self.first = True
        self.button1 = True
        self.button2 = True

    def click(self):

        self.clickPosition = mc.draggerContext(self.draggerName, query=True, anchorPoint=True)
        self.mouseButton = mc.draggerContext(self.draggerName, query=True, button=True)
        # LEFT mouse click to MIDDLE mouse click switching logic
        # LEFT mouse click
        if self.mouseButton == 1:

            # If this is not the first click of this button, and the previous button was the other button
            if self.first == False and self.mouseButton2 == True:
                # Then refresh the default value
                pass
            self.mouseButton1 = True
            self.mouseButton2 = False

        # RIGHT mouse click
        if self.mouseButton == 2:

            # If this is not the first click of this button, and the previous button was the other button
            if self.first == False and self.mouseButton1 == True:
                # Then refresh the default value
                pass
            self.mouseButton1 = False
            self.mouseButton2 = True

        self.first = True
        self.drag()

    # Procedure called on mouse drag
    def drag(self):

        if self.mouseButton == 1:
            limit = 100.0
        if self.mouseButton == 2:
            limit = 200.0

        # modifier = mc.draggerContext(self.draggerName, query=True, modifier=True)
        if self.first == True:
            self.dragPosition = self.clickPosition
            self.first = False
        else:
            self.dragPosition = mc.draggerContext(self.draggerName, query=True, dragPoint=True)

        # Find the difference between where you clicked and where you dragged then ADD it to the existince distance
        self.predistance = self.dragPosition[0] - self.clickPosition[0] + self.previousDistance

        # Then subtract
        self.distance = self.predistance - self.subtract - self.add
        self.difference = (self.distance * self.sensitivity)

        # This allows you to set a limit to how high/low the bar goes
        # Say you go to 150, when you subtract 3 you'll then get 97 instead of 147
        # If you go above 100, subtract the difference on next cycle
        if self.difference > limit:
            self.subtract = self.predistance - limit / self.sensitivity
            self.add = 0
            self.difference = limit

        # If you go below -100, add the difference to the next cycle
        if self.difference < -limit:
            self.add = self.predistance + limit / self.sensitivity
            self.subtract = 0
            self.difference = -limit

        self.distance = self.predistance - self.subtract - self.add

        self.percentage = self.difference
        self.nestedClass.tween(self.percentage, scrub=False)

    # Proedure called on mouse release
    def clickRelease(self):
        # mc.refresh(su=False)
        self.add = 0
        self.subtract = 0
        self.previousDistance = self.distance
        self.nestedClass.tween(self.percentage, scrub=True)
        mc.currentTime(mc.currentTime(q=True))

    # Called on button press
    def buttonPress(self, sensitivity, barLength, barOffset):
        self.draggerName = mc.draggerContext(name='testTool',
                                             pressCommand=self.click,
                                             dragCommand=self.drag,
                                             releaseCommand=self.clickRelease,
                                             undoMode='sequence',
                                             cursor='track')

        self.sensitivity = sensitivity
        self.currentTool = mc.currentCtx()
        self.nestedClass.settings(barLength, barOffset)
        self.nestedClass.refreshSelection()
        mc.setToolTo(self.draggerName)

    # Called on button release
    def buttonRelease(self):
        if self.currentTool != 'selectSuperContext' and self.currentTool != 'moveSuperContext' and self.currentTool != 'RotateSuperContext' and self.currentTool != 'scaleSuperContext':
            self.currentTool = 'moveSuperContext'
        mc.setToolTo(self.currentTool)

    def buttonPressMayaRestore(self, sensitivity, barLength, barOffset):
        mel.eval("storeLastAction( " + 'restoreLastContext ' + " + `currentCtx` )")
        self.buttonPress(sensitivity, barLength, barOffset)

    def buttonReleaseMayaRestore(self):
        mel.eval("invokeLastAction")

    def close(self):
        mc.delete(self.draggerName)


def press(sensitivity=1, barLength=100, barOffset=-200):
    global dragTweenObject
    dragTweenObject = dragTween()
    dragTweenObject.buttonPress(sensitivity, barLength, barOffset)


def release():
    dragTweenObject.buttonRelease()


# HAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHA
#  _____ _   _  _____ _______       _      _      
# |_   _| \ | |/ ____|__   __|/\   | |    | |     
#   | | |  \| | (___    | |  /  \  | |    | |     
#   | | | . ` |\___ \   | | / /\ \ | |    | |     
#  _| |_| |\  |____) |  | |/ ____ \| |____| |____ 
# |_____|_| \_|_____/   |_/_/    \_\______|______|
#                                                 
# HAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHAHA


class addHotkeyClass:

    def __init__(self):
        self.windowTitle = 'Add Tween Hotkey'
        self.windowName = 'tweenMenuUI'
        self.annotationPress = 'Press command for the drag tween functionality'
        self.annotationRelease = 'Release command for the drag tween functionality'
        self.sensitivity = .5
        self.height = -200
        self.length = 100

    def querySet(self):
        currentSet = mc.hotkeySet(q=True, current=True)
        if currentSet == "Maya_Default":
            mc.error('Cannot add hotkey because the current hotkey set is the Maya Default set')
        else:
            return currentSet

    def parseHotkey(self, hotkey):
        # Given a desired hotkey to use, find out the string that maya requires
        alphabet = ['a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z']
        numbers = ['0','1','2','3','4','5','6','7','8','9']
        special = ['/','*', '-', '+', ',', "'", ';', '.', '\\', '`', '[', ']']
        extraSpecial = ['Up', 'Down', 'Right', 'Left',
                'Home', 'End', 'Page_Up', 'Page_Down', 'Insert',
                'Return', 'Space',
                'F1', 'F2', 'F3', 'F4', 'F5', 'F6', 'F7', 'F8', 'F9', 'F10', 'F11', 'F12']
        specialModifierOnly = ['Tab', 'Delete', 'Backspace']

        if hotkey in special or hotkey in numbers or hotkey in extraSpecial:
            return hotkey
        if hotkey.lower() in alphabet:
            return hotkey.lower()

        mc.error('Provided hotkey is not valid, try a manual installation')

    def queryHotkey(self, hotkey):
        default = mc.hotkey(hotkey, query=True)

    def addPressNameCommand(self, commandName):
        pressCommand = "import triDragTween \ntriDragTween.press(sensitivity={}, barLength={}, barOffset={})".format(self.sensitivity, self.length, self.height)

        if mc.runTimeCommand(commandName, exists=True, q=True):
            mc.runTimeCommand(commandName, delete=True, e=True)

        self.pressRuntimeCommand = mc.runTimeCommand(commandName
                        ,label=""
                        ,image=""
                        ,tags=""
                        ,keywords=""
                        ,plugin=""
                        ,annotation=self.annotationPress
                        ,category="User"
                        ,hotkeyCtx=""
                        ,commandLanguage="python"
                        ,command=pressCommand)
        tempName = self.pressRuntimeCommand + 'NameCommand'
        self.pressNameCommand = mc.nameCommand(tempName, annotation=tempName, command=self.pressRuntimeCommand)

    def addReleaseNameCommand(self, commandName):
        releaseCommand = "import triDragTween \ntriDragTween.release()"
        if mc.runTimeCommand(commandName, exists=True, q=True):
            mc.runTimeCommand(commandName, delete=True, e=True)

        self.releaseRuntimeCommand = mc.runTimeCommand(commandName
                        ,label=""
                        ,image=""
                        ,tags=""
                        ,keywords=""
                        ,plugin=""
                        ,annotation=self.annotationRelease
                        ,category="User"
                        ,hotkeyCtx=""
                        ,commandLanguage="python"
                        ,command=releaseCommand)
        tempName = self.releaseRuntimeCommand + 'NameCommand'
        self.releaseNameCommand = mc.nameCommand(tempName, annotation=tempName, command=self.releaseRuntimeCommand)

    def assignHotkey(self, hotkey):
        mc.hotkey(keyShortcut=hotkey, name=self.pressNameCommand)
        mc.hotkey(keyShortcut=hotkey, releaseName=self.releaseNameCommand)

    def checkDialoug(self):
        result = mc.confirmDialog( title='Confirm', message='Overrite existing hotkey?', button=['Yes','No'], defaultButton='Yes', cancelButton='No', dismissString='No')
        if result != 'Yes':
            sys.exit()

    def modifiersCheck(self, *args):
        value = mc.checkBox(self.modifiersBox, value=True, q=True)
        self.modifiers = value

    def addHotkey(self, *args):
        self.querySet()
        self.checkDialoug()
        hotkeyRaw = str(mc.textField(self.hotkeyField, text=True, q=True))
        hotkey = self.parseHotkey(hotkeyRaw)
        self.sensitivity = mc.floatField(self.sensitivityField, q=True, value=True)
        self.length = mc.intField(self.lengthField, q=True, value=True)
        self.height = mc.intField(self.heightField, q=True, value=True)
        self.addPressNameCommand('triDragTweenPress')
        self.addReleaseNameCommand('triDragTweenRelease')
        self.assignHotkey(hotkey)
        self.close()

    def createUI(self):
        self.window = mc.window(self.windowName, toolbox=1, title=self.windowTitle)
        self.topLayout = mc.columnLayout(rowSpacing=10, columnAttach=('both', 5), columnAlign='right', adjustableColumn=1)

        mc.separator(style='none')

        mc.rowLayout(numberOfColumns=2)
        mc.text(label='Hotkey')
        self.hotkeyField = mc.textField(enable=True, editable=True, width=90)
        mc.setParent('..')

        mc.rowLayout(numberOfColumns=2)
        mc.text(label='Sensitivity')
        self.sensitivityField = mc.floatField(enable=True, value=.5, precision=2, editable=True, width=90)
        mc.setParent('..')

        mc.rowLayout(numberOfColumns=2)
        mc.text(label='Bar Length')
        self.lengthField = mc.intField(enable=True, value=100, maxValue=100, minValue=10, editable=True, width=90)
        mc.setParent('..')

        mc.rowLayout(numberOfColumns=2)
        mc.text(label='Bar Height')
        self.heightField = mc.intField(enable=True, value=-200, editable=True, width=90)
        mc.setParent('..')

        mc.rowLayout(numberOfColumns=2)
        mc.button(label="Assign", width=70, ann="Assign tweening tool to hotkey", command=self.addHotkey)
        mc.button(label="Cancel", width=70, ann="Cancel operation", command=self.close)

        mc.window(self.windowName, wh=[157, 170], sizeable=False, e=True)
        mc.showWindow(self.windowName)

    def run(self):
        if mc.window(self.windowName, exists=True):
            self.close()
        else:
            self.createUI()

    def close(self, *args):
        mc.deleteUI(self.windowName)

def install():
    temp = addHotkeyClass()
    temp.run()